@@ -12,7 +12,6 @@ import mediafile
 import time
 
 from datetime import datetime
-from pony.orm import db_session
 from queue import Queue, Empty as QueueEmpty
 from threading import Thread, Event
 
@@ -102,10 +101,10 @@ class Scanner(Thread):
             except QueueEmpty:
                 break
 
-            with db_session:
+            try:
                 folder = Folder.get(name=folder_name, root=True)
-                if folder is None:
-                    continue
+            except Folder.DoesNotExist:
+                continue
 
             self.__scan_folder(folder)
 
@@ -142,35 +141,36 @@ class Scanner(Thread):
 
                     self.__report_progress(folder.name, scanned)
 
-        # Remove files that have been deleted
-        # Could be more efficient if done above
-        if not self.__stopped.is_set():
-            with db_session:
-                for track in Track.select(lambda t: t.root_folder == folder):
-                    if not os.path.exists(track.path) or not self.__check_extension(
-                        track.path
-                    ):
-                        self.remove_file(track.path)
-
-        # Remove deleted/moved folders and update cover art info
+        # Remove deleted/moved folders
         folders = [folder]
         while not self.__stopped.is_set() and folders:
             f = folders.pop()
 
-            with db_session:
-                # f has been fetched from another session, refetch or Pony will complain
-                f = Folder[f.id]
+            if not f.root and not os.path.isdir(f.path):
+                self.__stats.deleted.tracks += f.delete_hierarchy()
+                continue
 
-                if not f.root and not os.path.isdir(f.path):
-                    f.delete()  # Pony will cascade
-                    continue
+            folders += f.children[:]
 
-                self.find_cover(f.path)
-                folders += f.children
+        # Remove files that have been deleted
+        # Could be more efficient if done when walking on the files
+        if not self.__stopped.is_set():
+            for track in Track.select().where(Track.root_folder == folder):
+                if not os.path.exists(track.path) or not self.__check_extension(
+                    track.path
+                ):
+                    self.remove_file(track.path)
+
+        # Update cover art info
+        folders = [folder]
+        while not self.__stopped.is_set() and folders:
+            f = folders.pop()
+            self.find_cover(f.path)
+            folders += f.children[:]
 
         if not self.__stopped.is_set():
-            with db_session:
-                Folder[folder.id].last_scan = int(time.time())
+            folder.last_scan = int(time.time())
+            folder.save()
 
         if self.__on_folder_end is not None:
             self.__on_folder_end(folder)
@@ -179,17 +179,15 @@ class Scanner(Thread):
         if self.__stopped.is_set():
             return
 
-        with db_session:
-            self.__stats.deleted.albums = Album.prune()
-            self.__stats.deleted.artists = Artist.prune()
-            Folder.prune()
+        self.__stats.deleted.albums += Album.prune()
+        self.__stats.deleted.artists += Artist.prune()
+        Folder.prune()
 
     def __check_extension(self, path):
         if not self.__extensions:
             return True
         return os.path.splitext(path)[1][1:].lower() in self.__extensions
 
-    @db_session
     def scan_file(self, path_or_direntry):
         if isinstance(path_or_direntry, str):
             path = path_or_direntry
@@ -212,7 +210,7 @@ class Scanner(Thread):
 
         mtime = int(stat.st_mtime)
 
-        tr = Track.get(path=path)
+        tr = Track.get_or_none(path=path)
         if tr is not None:
             if not self.__force and not mtime > tr.last_modification:
                 return
@@ -241,7 +239,7 @@ class Scanner(Thread):
         trdict["duration"] = int(tag.length)
         trdict["has_art"] = bool(tag.images)
 
-        trdict["bitrate"] = tag.bitrate
+        trdict["bitrate"] = tag.bitrate // 1000
         trdict["last_modification"] = mtime
 
         tralbum = self.__find_album(albumartist, album)
@@ -255,7 +253,7 @@ class Scanner(Thread):
             trdict["created"] = datetime.fromtimestamp(mtime)
 
             try:
-                Track(**trdict)
+                Track.create(**trdict)
                 self.__stats.added.tracks += 1
             except ValueError:
                 # Field validation error
@@ -268,24 +266,23 @@ class Scanner(Thread):
                 trdict["artist"] = trartist
 
             try:
-                tr.set(**trdict)
+                for attr, value in trdict.items():
+                    setattr(tr, attr, value)
+                tr.save()
             except ValueError:
                 # Field validation error
                 self.__stats.errors.append(path)
 
-    @db_session
     def remove_file(self, path):
         if not isinstance(path, str):
             raise TypeError("Expecting string, got " + str(type(path)))
 
-        tr = Track.get(path=path)
-        if not tr:
-            return
+        try:
+            Track.get(path=path).delete_instance(recursive=True)
+            self.__stats.deleted.tracks += 1
+        except Track.DoesNotExist:
+            pass
 
-        self.__stats.deleted.tracks += 1
-        tr.delete()
-
-    @db_session
     def move_file(self, src_path, dst_path):
         if not isinstance(src_path, str):
             raise TypeError("Expecting string, got " + str(type(src_path)))
@@ -295,25 +292,26 @@ class Scanner(Thread):
         if src_path == dst_path:
             return
 
-        tr = Track.get(path=src_path)
-        if tr is None:
+        try:
+            tr = Track.get(path=src_path)
+        except Track.DoesNotExist:
             return
 
-        tr_dst = Track.get(path=dst_path)
-        if tr_dst is not None:
+        try:
+            tr_dst = Track.get(path=dst_path)
             root = tr_dst.root_folder
             folder = tr_dst.folder
             self.remove_file(dst_path)
             tr.root_folder = root
             tr.folder = folder
-        else:
+        except Track.DoesNotExist:
             root = self.__find_root_folder(dst_path)
             folder = self.__find_folder(dst_path)
             tr.root_folder = root
             tr.folder = folder
         tr.path = dst_path
+        tr.save()
 
-    @db_session
     def find_cover(self, dirpath):
         if not isinstance(dirpath, str):  # pragma: nocover
             raise TypeError("Expecting string, got " + str(type(dirpath)))
@@ -321,8 +319,9 @@ class Scanner(Thread):
         if not os.path.exists(dirpath):
             return
 
-        folder = Folder.get(path=dirpath)
-        if folder is None:
+        try:
+            folder = Folder.get(path=dirpath)
+        except Folder.DoesNotExist:
             return
 
         album_name = None
@@ -332,19 +331,21 @@ class Scanner(Thread):
 
         cover = find_cover_in_folder(folder.path, album_name)
         folder.cover_art = cover.name if cover is not None else None
+        folder.save()
 
-    @db_session
     def add_cover(self, path):
         if not isinstance(path, str):  # pragma: nocover
             raise TypeError("Expecting string, got " + str(type(path)))
 
-        folder = Folder.get(path=os.path.dirname(path))
-        if folder is None:
+        try:
+            folder = Folder.get(path=os.path.dirname(path))
+        except Folder.DoesNotExist:
             return
 
         cover_name = os.path.basename(path)
         if not folder.cover_art:
             folder.cover_art = cover_name
+            folder.save()
         elif folder.cover_art != cover_name:
             album_name = None
             track = folder.tracks.select().first()
@@ -355,31 +356,27 @@ class Scanner(Thread):
             new_cover = CoverFile(cover_name, album_name)
             if new_cover.score > current_cover.score:
                 folder.cover_art = cover_name
+                folder.save()
 
     def __find_album(self, artist, album):
         ar = self.__find_artist(artist)
-        al = ar.albums.select(lambda a: a.name == album).first()
+        al = ar.albums.where(Album.name == album).first()
         if al:
             return al
 
-        al = Album(name=album, artist=ar)
         self.__stats.added.albums += 1
-
-        return al
+        return Album.create(name=album, artist=ar)
 
     def __find_artist(self, artist):
-        ar = Artist.get(name=artist)
-        if ar:
-            return ar
-
-        ar = Artist(name=artist)
-        self.__stats.added.artists += 1
-
-        return ar
+        try:
+            return Artist.get(name=artist)
+        except Artist.DoesNotExist:
+            self.__stats.added.artists += 1
+            return Artist.create(name=artist)
 
     def __find_root_folder(self, path):
         path = os.path.dirname(path)
-        for folder in Folder.select(lambda f: f.root):
+        for folder in Folder.select().where(Folder.root):
             if path.startswith(folder.path):
                 return folder
 
@@ -394,9 +391,11 @@ class Scanner(Thread):
         drive, _ = os.path.splitdrive(path)
         path = os.path.dirname(path)
         while path not in (drive, "/"):
-            folder = Folder.get(path=path)
-            if folder is not None:
+            try:
+                folder = Folder.get(path=path)
                 break
+            except Folder.DoesNotExist:
+                pass
 
             created = datetime.fromtimestamp(os.path.getmtime(path))
             children.append(
@@ -411,7 +410,7 @@ class Scanner(Thread):
 
         assert folder is not None
         while children:
-            folder = Folder(parent=folder, **children.pop())
+            folder = Folder.create(parent=folder, **children.pop())
 
         return folder
 
